@@ -137,6 +137,16 @@ class SymbolicSys(ODESys):
             do not compute jacobian (use explicit steppers)
         if instance of ImmutableMatrix:
             user provided expressions for the jacobian
+    jtimes : bool or iterable of (symbol, expression)-pairs
+        if True:
+            calculate jacobian-vector product from exprs
+        if False:
+            do not compute jacobian-vector product
+        if instance of iterable of (symbol, expression) pairs:
+            user-provided expressions for the jacobian-vector product, where the symbols
+            are the elements of the vector being multiplied. Other than these symbols,
+            the expressions should only involve the symbols in :attr`dep` and
+            :attr`params`.
     dfdx : iterable of expressions
         Derivatives of :attr:`exprs` with respect to :attr`indep`.
     first_step_expr : expression
@@ -167,7 +177,11 @@ class SymbolicSys(ODESys):
         When ``True`` construct using ``be.Symbol``. See also :attr:`init_indep`.
     init_dep : tuple of Symbols, ``True`` or ``None``
         When ``True`` construct using ``be.Symbol``. See also :attr:`init_dep`.
-    \*\*kwargs:
+    sparse : bool (default ``False``)
+        When ``True``, jacobian will be derived and stored (if ``jac = True``)
+        in compressed sparse column (CSC) format and the jacobian callback
+        will return an instance of ``scipy.sparse.csc_matrix``.
+    \\*\\*kwargs:
         See :py:class:`ODESys`
 
     Attributes
@@ -214,11 +228,11 @@ class SymbolicSys(ODESys):
     def linear_invariants(self, lin_invar):
         self._linear_invariants = _get_lin_invar_mtx(lin_invar, self.be, self.ny, self.names)
 
-    def __init__(self, dep_exprs, indep=None, params=None, jac=True, dfdx=True, first_step_expr=None,
-                 roots=None, backend=None, lower_bounds=None, upper_bounds=None,
-                 linear_invariants=None, nonlinear_invariants=None,
+    def __init__(self, dep_exprs, indep=None, params=None, jac=True, dfdx=True,
+                 jtimes=False, first_step_expr=None, roots=None, backend=None, lower_bounds=None,
+                 upper_bounds=None, linear_invariants=None, nonlinear_invariants=None,
                  linear_invariant_names=None, nonlinear_invariant_names=None, steady_state_root=False,
-                 init_indep=None, init_dep=None, **kwargs):
+                 init_indep=None, init_dep=None, sparse=False, **kwargs):
         self.dep, self.exprs = zip(*dep_exprs.items()) if isinstance(dep_exprs, dict) else zip(*dep_exprs)
         self.indep = indep
         if params is True or params is None:
@@ -230,9 +244,11 @@ class SymbolicSys(ODESys):
 
         self.params = tuple(params)
         self._jac = jac
+        self._jtimes = tuple(jtimes) if hasattr(jtimes, '__iter__') else jtimes
         self._dfdx = dfdx
         self.first_step_expr = first_step_expr
         self.be = Backend(backend)
+
         if steady_state_root:
             if steady_state_root is True:
                 steady_state_root = 1e-10
@@ -266,6 +282,7 @@ class SymbolicSys(ODESys):
         if _param_names is True:
             kwargs['param_names'] = [p.name for p in self.params]
 
+        self.sparse = sparse  # needed by get_j_ty_callback
         self.band = kwargs.get('band', None)  # needed by get_j_ty_callback
         # bounds needed by get_f_ty_callback:
         self.lower_bounds = None if lower_bounds is None else np.array(lower_bounds)*np.ones(self.ny)
@@ -273,14 +290,17 @@ class SymbolicSys(ODESys):
 
         super(SymbolicSys, self).__init__(
             self.get_f_ty_callback(),
-            self.get_j_ty_callback(),
-            self.get_dfdx_callback(),
-            self.get_first_step_callback(),
-            self.get_roots_callback(),
+            jac=self.get_j_ty_callback(),
+            dfdx=self.get_dfdx_callback(),
+            jtimes=self.get_jtimes_callback(),
+            first_step_cb=self.get_first_step_callback(),
+            roots_cb=self.get_roots_callback(),
             nroots=None if roots is None else len(roots),
             autonomous_exprs=_is_autonomous(self.indep, self.exprs),
             **kwargs)
 
+        if self.sparse:
+            self.nnz = len(self._rowvals)
         self.linear_invariants = linear_invariants
         self.nonlinear_invariants = nonlinear_invariants
         self.linear_invariant_names = linear_invariant_names or None
@@ -385,7 +405,7 @@ class SymbolicSys(ODESys):
             its return value from dict to array.
         par_by_name : bool
             Make ``p`` passed to ``rhs`` a dict (keys from :attr:`param_names`).
-        \*\*kwargs :
+        \\*\\*kwargs :
             Keyword arguments passed onto :class:`SymbolicSys`.
 
         Examples
@@ -521,7 +541,7 @@ class SymbolicSys(ODESys):
         drop_idxs = [ori.params.index(par) for par in par_subs]
         params = _skip(drop_idxs, ori.params, False) + list(new_pars)
         back_substitute = _Callback(ori.indep, ori.dep, params, list(par_subs.values()),
-                                    ori.be.Lambdify)
+                                    Lambdify=ori.be.Lambdify)
 
         def recalc_params(t, y, p):
             rev = back_substitute(t, y, p)
@@ -617,16 +637,32 @@ class SymbolicSys(ODESys):
     def get_jac(self):
         """ Derives the jacobian from ``self.exprs`` and ``self.dep``. """
         if self._jac is True:
-            if self.band is None:
+            if self.sparse is True:
+                self._jac, self._colptrs, self._rowvals = self.be.sparse_jacobian_csc(self.exprs, self.dep)
+            elif self.band is not None:  # Banded
+                self._jac = self.be.banded_jacobian(self.exprs, self.dep, *self.band)
+            else:
                 f = self.be.Matrix(1, self.ny, self.exprs)
                 self._jac = f.jacobian(self.be.Matrix(1, self.ny, self.dep))
-            else:  # Banded
-                self._jac = self.be.banded_jacobian(self.exprs, self.dep, *self.band)
         elif self._jac is False:
             return False
 
         return self._jac
 
+    def get_jtimes(self):
+        """ Derive the jacobian-vector product from ``self.exprs`` and ``self.dep``"""
+        if self._jtimes is False:
+            return False
+
+        if self._jtimes is True:
+            r = self.be.Dummy('r')
+            v = tuple(self.be.Dummy('v_{0}'.format(i)) for i in range(self.ny))
+            f = self.be.Matrix(1, self.ny, self.exprs)
+            f = f.subs([(x_i, x_i + r * v_i) for x_i, v_i in zip(self.dep, v)])
+            return v, self.be.flatten(f.diff(r).subs(r, 0))
+        else:
+            return tuple(zip(*self._jtimes))
+
     def jacobian_singular(self):
         """ Returns True if Jacobian is singular, else False. """
         cses, (jac_in_cses,) = self.be.cse(self.get_jac())
@@ -648,7 +684,7 @@ class SymbolicSys(ODESys):
         return self._dfdx
 
     def _callback_factory(self, exprs):
-        return _Callback(self.indep, self.dep, self.params, exprs, self.be.Lambdify)
+        return _Callback(self.indep, self.dep, self.params, exprs, Lambdify=self.be.Lambdify)
 
     def get_f_ty_callback(self):
         """ Generates a callback for evaluating ``self.exprs``. """
@@ -677,7 +713,17 @@ class SymbolicSys(ODESys):
         j_exprs = self.get_jac()
         if j_exprs is False:
             return None
-        return self._callback_factory(j_exprs)
+        cb = self._callback_factory(j_exprs)
+        if self.sparse:
+            from scipy.sparse import csc_matrix
+
+            def sparse_cb(x, y, p=()):
+                data = cb(x, y, p).flatten()
+                return csc_matrix((data, self._rowvals, self._colptrs))
+
+            return sparse_cb
+        else:
+            return cb
 
     def get_dfdx_callback(self):
         """ Generate a callback for evaluating derivative of ``self.exprs`` """
@@ -686,6 +732,15 @@ class SymbolicSys(ODESys):
             return None
         return self._callback_factory(dfdx_exprs)
 
+    def get_jtimes_callback(self):
+        """ Generate a callback fro evaluating the jacobian-vector product."""
+        jtimes = self.get_jtimes()
+        if jtimes is False:
+            return None
+        v, jtimes_exprs = jtimes
+        return _Callback(self.indep, tuple(self.dep) + tuple(v), self.params,
+                         jtimes_exprs, Lambdify=self.be.Lambdify)
+
     def get_first_step_callback(self):
         if self.first_step_expr is None:
             return None
@@ -763,7 +818,7 @@ class TransformedSys(SymbolicSys):
     check_transforms : bool
         Passed as keyword argument ``check`` to :func:`.util.transform_exprs_dep` and
         :func:`.util.transform_exprs_indep`.
-    \*\*kwargs :
+    \\*\\*kwargs :
         Keyword arguments passed onto :class:`SymbolicSys`.
 
     """
@@ -849,7 +904,7 @@ class TransformedSys(SymbolicSys):
         roots_cb : callable
             Callback with signature ``roots(x, y[:], p[:], backend=math) -> r[:]``.
             Callback should return untransformed roots.
-        \*\*kwargs :
+        \\*\\*kwargs :
             Keyword arguments passed onto :class:`TransformedSys`.
 
         """
@@ -917,7 +972,7 @@ def symmetricsys(dep_tr=None, indep_tr=None, SuperClass=TransformedSys, **kwargs
         Forward and backward transformation to be applied to the
         independent variable.
     SuperClass : class
-    \*\*kwargs :
+    \\*\\*kwargs :
         Default keyword arguments for the TransformedSys subclass.
 
     Returns
@@ -987,7 +1042,7 @@ class ScaledSys(TransformedSys):
         scaling of the independent variable (default: 1)
     params :
         see :class:`SymbolicSys`
-    \*\*kwargs :
+    \\*\\*kwargs :
         Keyword arguments passed onto :class:`TransformedSys`.
 
     Examples
@@ -1047,7 +1102,7 @@ class ScaledSys(TransformedSys):
             scaling of the dependent variables (default: 1)
         indep_scaling: number (>0)
             scaling of the independent variable (default: 1)
-        \*\*kwargs :
+        \\*\\*kwargs :
             Keyword arguments passed onto :class:`ScaledSys`.
 
         Examples
@@ -1088,7 +1143,7 @@ class PartiallySolvedSystem(SymbolicSys):
         the signature: ``my_factory(x0, y0, p0, backend) -> dict``, where the returned
         dictionary maps dependent variabels (from ``original_system.dep``)
         to new expressions in remaining variables and initial conditions.
-    \*\*kwargs : dict
+    \\*\\*kwargs : dict
         Keyword arguments passed onto :class:`SymbolicSys`.
 
     Attributes
@@ -1230,7 +1285,7 @@ class PartiallySolvedSystem(SymbolicSys):
         preferred : iterable of preferred dependent variables
             Due to numerical rounding it is preferable to choose the variables
             which are expected to be of the largest magnitude during integration.
-        \*\*kwargs :
+        \\*\\*kwargs :
             Keyword arguments passed on to constructor.
         """
         _be = ori_sys.be
@@ -1281,7 +1336,7 @@ class PartiallySolvedSystem(SymbolicSys):
 
     @staticmethod
     def _get_analytic_callback(ori_sys, analytic_exprs, new_dep, new_params):
-        return _Callback(ori_sys.indep, new_dep, new_params, analytic_exprs, ori_sys.be.Lambdify)
+        return _Callback(ori_sys.indep, new_dep, new_params, analytic_exprs, Lambdify=ori_sys.be.Lambdify)
 
     def __getitem__(self, key):
         ori_dep = self.original_dep[self.names.index(key)]
