@@ -1,8 +1,6 @@
 # -*- coding: utf-8 -*-
 from __future__ import (absolute_import, division, print_function)
 
-import math
-
 import numpy as np
 
 from .. import ODESys
@@ -17,11 +15,17 @@ def _test_sine(use_deriv, atol=1e-8, rtol=1e-8, forgive=1e4):
                               atol=atol, rtol=rtol)
     x_probe = 2**-0.5
     assert result.info['success']
-    ref = np.array([A*math.sin(k*x_probe), A*math.cos(k*x_probe)*k])
+
+    ref = lambda x: np.array([A*np.sin(k*x), A*np.cos(k*x)*k]).T
     est, est_err = result.at(x_probe, use_deriv=use_deriv)
-    real_err = np.abs(est - ref)
+    real_err = np.abs(est - ref(x_probe))
     assert np.all(real_err < est_err)
-    assert np.allclose(ref, est, atol=atol*forgive, rtol=rtol*forgive)
+    assert np.allclose(ref(x_probe), est, atol=atol*forgive, rtol=rtol*forgive)
+
+    probes = np.linspace(0, 1, 11)
+    est2, est_err2 = result.at(probes, linear=True)
+    assert est_err2 is None
+    assert np.allclose(ref(probes), est2, atol=3e-2)
 
 
 @requires('scipy')
