@@ -6,9 +6,13 @@ from collections import defaultdict, OrderedDict
 from itertools import product
 import math
 
+try:
+    from time import clock as perf_counter
+except ImportError:
+    from time import perf_counter
+
 import numpy as np
 import pytest
-import time
 
 try:
     import sym
@@ -21,7 +25,7 @@ else:
 from .. import ODESys
 from ..core import integrate_auto_switch, chained_parameter_variation
 from ..symbolic import SymbolicSys, ScaledSys, symmetricsys, PartiallySolvedSystem, get_logexp, _group_invariants
-from ..util import requires
+from ..util import requires, pycvodes_double, pycvodes_klu
 from .bateman import bateman_full  # analytic, never mind the details
 from .test_core import vdp_f
 from . import _cetsa
@@ -136,6 +140,18 @@ def test_SymbolicSys__jacobian_singular():
     assert odesys.jacobian_singular()
 
 
+@requires('sym', 'pycvodes')
+@pycvodes_klu
+def test_SymbolicSys_jacobian_sparse():
+    k = (4, 3)
+    y0 = (5, 4, 2)
+    odesys = SymbolicSys.from_callback(decay_dydt_factory(k), len(k) + 1, sparse=True)
+    xout, yout, info = odesys.integrate([0, 2], y0, integrator='cvode', linear_solver='klu',
+                                        atol=1e-12, rtol=1e-12, nsteps=1000)
+    ref = np.array(bateman_full(y0, k+(0,), xout - xout[0], exp=np.exp)).T
+    assert np.allclose(yout, ref, rtol=1e-10, atol=1e-10)
+
+
 @requires('sym', 'pycvodes')
 def test_TransformedSys_liny_linx():
     _test_TransformedSys(idty2, idty2, 1e-11, 1e-11, 0, 15)
@@ -250,9 +266,9 @@ def test_ScaledSys_from_callback__exprs():
 
 
 def timeit(callback, *args, **kwargs):
-    t0 = time.clock()
+    t0 = perf_counter()
     result = callback(*args, **kwargs)
-    return time.clock() - t0, result
+    return perf_counter() - t0, result
 
 
 @requires('sym', 'pyodeint')
@@ -1176,6 +1192,7 @@ def test_symmetricsys__invariants():
 
 
 @requires('sym', 'pycvodes')
+@pycvodes_double
 def test_SymbolicSys__roots():
     def f(t, y):
         return [y[0]]
@@ -1183,7 +1200,7 @@ def test_SymbolicSys__roots():
     def roots(t, y, p, backend):
         return [y[0] - backend.exp(1)]
     odesys = SymbolicSys.from_callback(f, 1, roots_cb=roots)
-    kwargs = dict(first_step=1e-12, atol=1e-12, rtol=1e-12, method='adams', integrator='cvode')
+    kwargs = dict(first_step=1e-14, atol=1e-14, rtol=1e-14, method='adams', integrator='cvode')
     xout, yout, info = odesys.integrate(2, [1], **kwargs)
     assert len(info['root_indices']) == 1
     assert np.min(np.abs(xout - 1)) < 1e-11
